/*
 * This file is part of architectury.
 * Copyright (C) 2020, 2021 architectury
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

package me.shedaniel.architectury.utils.fabric;

import me.shedaniel.architectury.event.EventHandler;
import me.shedaniel.architectury.event.events.LifecycleEvent;
import me.shedaniel.architectury.platform.Platform;
import me.shedaniel.architectury.utils.Env;
import net.fabricmc.api.EnvType;
import net.fabricmc.api.Environment;
import net.minecraft.client.Minecraft;
import net.minecraft.server.MinecraftServer;

public class GameInstanceImpl {
    private static MinecraftServer server = null;
    
    public static MinecraftServer getServer() {
        MinecraftServer server = null;
        if (GameInstanceImpl.server != null) server = GameInstanceImpl.server;
        if (Platform.getEnvironment() == Env.CLIENT) {
            server = getServerFromClient();
        }
        return server;
    }
    
    public static void init() {
        EventHandler.init();
        LifecycleEvent.SERVER_BEFORE_START.register(server -> GameInstanceImpl.server = server);
        LifecycleEvent.SERVER_STOPPED.register(server -> GameInstanceImpl.server = null);
    }
    
    @Environment(EnvType.CLIENT)
    private static MinecraftServer getServerFromClient() {
        return Minecraft.getInstance().getSingleplayerServer();
    }
}
